package com.me.pacman.state;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Input;
import com.badlogic.gdx.InputAdapter;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.g2d.BitmapFont;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.me.pacman.PacDude;
import com.me.pacman.entity.Direction;
import com.me.pacman.entity.Ghost;
import com.me.pacman.entity.Pacman;
import com.me.pacman.entity.ai.BlinkyChaseBehaviour;
import com.me.pacman.level.Level;
import com.me.pacman.level.LevelTile;

import java.util.Random;

public class PlayState extends LevelState {

    private Texture levelBackground;
    private BitmapFont font;

    private TextureRegion lifeSprite;

    private int pelletCount;
    private int pelletEatenCount;
    private int score;
    private int lives;
    private int round;
    private boolean paused = false;

    private long sirenId;

    private float readyTimer;
    private float gameOverTimer;
    private float newGameTimer;
    private float deathTimer;

    private boolean pacmanCaught;

    public Random random;

    public Pacman pacman;
    public Ghost[] ghosts;

    public PlayState(PacDude game) {
        super(game);
    }

    @Override
    public void setup() {
        levelBackground = game.assets.getLevelBackground();
        font = game.assets.getFont();
        lifeSprite = game.assets.pacman[2][1];
        Gdx.input.setInputProcessor(new Controller());

        newGame();
    }

    @Override
    public void render() {
        game.batch.draw(levelBackground, 0, 16);
        level.render(0, 16);

        game.assets.getFont().setColor(Color.WHITE);
        game.assets.getFont().draw(game.batch, "" + score, 40, 279);
        for (int i = 0; i < lives; i++) {
            game.batch.draw(lifeSprite, i * 16, 0);
        }

        pacman.render(game.batch, 0, 16);

        if (pacman.alive) {
            for (Ghost ghost : ghosts) {
                if (ghost == null) {
                    continue;
                }
                ghost.render(game.batch, 0, 16);
            }
        }

        if (readyTimer > 0) {
            game.assets.getFont().setColor(Color.YELLOW);
            game.assets.getFont().draw(game.batch, "ready!", 92, 127);
        }

        if (gameOverTimer > 0) {
            game.assets.getFont().setColor(Color.RED);
            game.assets.getFont().draw(game.batch, "game  over", 78, 127);
        }

        if (paused) {
            game.assets.getFont().setColor(Color.YELLOW);
            game.assets.getFont().draw(game.batch, "paused", 90, 151);
        }
    }

    @Override
    public void update(float dt) {
        if (paused) {
            return;
        }

        if (gameOverTimer > 0) {
            gameOverTimer -= dt;
            if (gameOverTimer <= 0) {
                newGame();
            } else {
                return;
            }
        }

        if (readyTimer > 0) {
            readyTimer -= dt;
            if (readyTimer <= 0) {
                startGame();
            } else if (newGameTimer <= 0) {
                return;
            }
        }

        if (newGameTimer > 0) {
            newGameTimer -= dt;
            if (newGameTimer <= 0) {
                setupGame();
            } else {
                return;
            }
        }

        dt = 1/60f;

        if (!pacmanCaught) {
            for (Ghost ghost : ghosts) {
                if (ghost == null) {
                    continue;
                }
                ghost.update(dt);

                if (ghost.onSameTile(pacman)) {
                    pacmanCaught();
                    return;
                }
            }
        } else if (deathTimer <= 0) {
            if (pacman.alive) {
                game.assets.deathSound.play(1f);
            }

            pacman.alive = false;

            if (pacman.deathFrame == 13) {
                if (lives > 0) {
                    setupGame();
                    readyTimer = 3f;
                } else {
                    gameOverTimer = 2f;
                }
            }
        } else {
            deathTimer -= dt;
        }

        pacman.update(dt);
    }

    public void newGame() {
        pelletCount = 0;
        pelletEatenCount = 0;
        score = 0;
        lives = 3;
        round = 0;

        pacmanCaught = false;
        deathTimer = 0f;

        newGameTimer = 2.3f;

        ghosts = new Ghost[4];
        pacman = new Pacman(this, false);

        newRound();
    }

    public void newRound() {
        round++;
        level = new Level(game,"level");

        pacman = new Pacman(this, false);
        ghosts[0] = new Ghost(this, 14, 19.5f, Direction.LEFT, 0, new BlinkyChaseBehaviour(this), null);
        ghosts[0].currentBehaviour = ghosts[0].chaseBehaviour;

        game.assets.siren.stop(sirenId);

        pelletCount = level.getTileCount(LevelTile.PELLET);
        pelletCount += level.getTileCount(LevelTile.POWER_PELLET);

        if (round == 1) {
            readyTimer = 4.2f;
            game.assets.beginning_alt.play(1.0f);
        } else {
            readyTimer = 4.3f;
            game.assets.beginning.play(1.0f);
        }
    }

    public void setupGame() {
        lives--;
        pacmanCaught = false;

        random = new Random(897198256012865L);

        pacman = new Pacman(this, false);
        ghosts[0] = new Ghost(this, 14, 19.5f, Direction.LEFT, 0, new BlinkyChaseBehaviour(this), null);
        ghosts[0].currentBehaviour = ghosts[0].chaseBehaviour;
    }

    public void startGame() {
        pacman.moving = true;
        sirenId = game.assets.siren.loop(1.0f);
    }

    private void pelletEaten(float x, float y) {
        level.setTile(x, y, LevelTile.EMPTY);
        if (pelletEatenCount % 2 == 0) {
            game.assets.chomp_1.play(1.0f);
        } else {
            game.assets.chomp_2.play(1.0f);
        }
        pelletEatenCount++;
        pelletCount--;

        if (pelletCount == 0) {
            newRound();
        }
    }

    public void eatPellet(float x, float y) {
        pelletEaten(x, y);
        score += 10;
    }

    public void eatPowerPellet(float x, float y) {
        pelletEaten(x, y);
        // TODO: Enable ghost chase mode
        score += 50;
    }

    private void pacmanCaught() {
        game.assets.siren.stop(sirenId);
        deathTimer = 1f;
        pacmanCaught = true;
        pacman.moving = false;
    }


    @Override
    public void dispose() {
        Gdx.input.setInputProcessor(null);
    }

    private final class Controller extends InputAdapter {

        @Override
        public boolean keyDown(int keycode) {
            switch (keycode) {
                case Input.Keys.UP:
                    pacman.setNextDirection(Direction.UP);
                    break;
                case Input.Keys.DOWN:
                    pacman.setNextDirection(Direction.DOWN);
                    break;
                case Input.Keys.LEFT:
                    pacman.setNextDirection(Direction.LEFT);
                    break;
                case Input.Keys.RIGHT:
                    pacman.setNextDirection(Direction.RIGHT);
                    break;
                case Input.Keys.P:
                    paused = !paused;
                    break;
            }
            return super.keyDown(keycode);
        }

    }

}
